-- -------------------------------------------------------------------------- --
-- GridIndicatorExtraText by kunda                                            --
-- -------------------------------------------------------------------------- --

local addonName, prg = ...
if type(prg)   ~= "table" then print("ERROR", addonName) return end
if type(prg.L) ~= "table" then print("ERROR", addonName, "No localization table.") return end
for k, v in pairs(prg.L) do if type(v) ~= "string" then prg.L[k] = tostring(k) end end
local L = setmetatable(prg.L, {
	__index = function(t, k)
		t[k] = k
		--print(addonName, "prg.L:", k)
		return k
	end
})

local Grid = Grid
if not Grid.L then Grid.L = {} end
local LGrid = setmetatable(Grid.L, {
	__index = function(t, k)
		t[k] = k
		--print(addonName, "Grid.L:", k)
		return k
	end
})
local GridFrame = Grid:GetModule("GridFrame")
local GridIndicatorExtraText = GridFrame:NewModule("GridIndicatorExtraText")

local configMode = false
local media = LibStub("LibSharedMedia-3.0")
local strsub = string.utf8sub or string.sub



local function NewIndicator(frame)
	local text = frame:CreateFontString(nil, "OVERLAY", "GameFontHighlightSmall")
	return text
end



local function ResetIndicator(self)
	local frame = self.__owner
	local indicator = self.__id
	local AddonProfile = GridIndicatorExtraText.db.profile
	local GridProfile = GridFrame.db.profile

	local fontType = media and media:Fetch("font", AddonProfile.fontType) or STANDARD_TEXT_FONT
	local fontSize = AddonProfile.fontSize
	local fontOutline = AddonProfile.fontOutline
	local fontShadow = AddonProfile.fontShadow
	if AddonProfile.OriginalSize then
		fontType = media and media:Fetch("font", GridProfile.fontType) or STANDARD_TEXT_FONT
		fontSize = GridProfile.fontSize
		fontOutline = GridProfile.fontOutline
		fontShadow = GridProfile.fontShadow
	end

	local bar = frame.indicators.bar

	self:SetParent(bar)
	self:SetFont(fontType, fontSize, fontOutline)

	if fontShadow then
		self:SetShadowOffset(1, -1)
	else
		self:SetShadowOffset(0, 0)
	end

	if GridProfile.invertBarColor and GridProfile.invertTextColor then
		self:SetShadowColor(1, 1, 1)
	else
		self:SetShadowColor(0, 0, 0)
	end

	self:SetJustifyH("LEFT")
	self:SetJustifyV("CENTER")

	self:ClearAllPoints()
	if     indicator == "extraText1TL" then self:SetPoint("TOPLEFT", bar, "TOPLEFT", AddonProfile.offset1H, -AddonProfile.offset1V)
	elseif indicator == "extraText2TR" then self:SetPoint("TOPRIGHT", bar, "TOPRIGHT", AddonProfile.offset2H, -AddonProfile.offset2V)
	elseif indicator == "extraText3BL" then self:SetPoint("BOTTOMLEFT", bar, "BOTTOMLEFT", AddonProfile.offset3H, -AddonProfile.offset3V)
	elseif indicator == "extraText4BR" then	self:SetPoint("BOTTOMRIGHT", bar, "BOTTOMRIGHT", AddonProfile.offset4H, -AddonProfile.offset4V)
	end
end



local function SetIndicator(self, color, text, value, maxValue, texture, texCoords, count, start, duration)
	local AddonProfile = GridIndicatorExtraText.db.profile
	local GridProfile = GridFrame.db.profile
	
	local maxLen = AddonProfile.maxTextLength
	if AddonProfile.OriginalSize then
		maxLen = GridProfile.textlength
	end

	self:SetText(strsub(text, 1, maxLen))

	if color then
		if GridProfile.invertBarColor and GridProfile.invertTextColor then
			self:SetTextColor(color.r * 0.2, color.g * 0.2, color.b * 0.2, color.a or 1)
		else
			self:SetTextColor(color.r, color.g, color.b, color.a or 1)
		end
	end
end



local function ClearIndicator(self)
	if configMode then return end
	self:SetText("")
end



local function SetCornerSize(frame)
	ResetIndicator(frame.indicators.extraText1TL)
	ResetIndicator(frame.indicators.extraText2TR)
	ResetIndicator(frame.indicators.extraText3BL)
	ResetIndicator(frame.indicators.extraText4BR)
end



GridIndicatorExtraText.defaultDB = {
	OriginalSize = false,
	maxTextLength = 3,
	fontType = "Friz Quadrata TT",
	fontSize = 8,
	fontOutline = "OUTLINE",
	fontShadow = false,
	offset1H = 1,
	offset1V = 0,
	offset2H = -1,
	offset2V = 0,
	offset3H = 1,
	offset3V = 0,
	offset4H = -1,
	offset4V = 0
}



local options = {
	type = "group",
	--inline = true,
	icon = "Interface\\AddOns\\GridIndicatorExtraText\\GridIndicatorExtraText-icon-ALL",
	name = L["Extra Text"],
	desc = format(LGrid["Options for %s indicator."], L["Extra Text"]),
	order = -0.551,--480,
	args = {
		["configuration"] = {
			type = "toggle",
			name = L["Configuration Mode"],
			order = 5,
			get = function() return configMode end,
			set = function(_, v) 
				configMode = v
				GridIndicatorExtraText:ConfigMode()
			end
		},
		["header1"] = {
			type = "header",
			order = 10,
			width = "full",
			name = "",
		},
		["desc1"] = {
			type = "description",
			order = 11,
			width = "full",
			name = L["Offset"].." "..LGrid["Top Left"]..":",
		},
		["offset1H"] = {
			type = "range",
			name = LGrid["Horizontal"],
			width = "fill",
			order = 12,
			min = -20,
			max = 20,
			step = 1,
			get = function() return GridIndicatorExtraText.db.profile.offset1H end,
			set = function(_, v)
				GridIndicatorExtraText.db.profile.offset1H = v
				for _, f in pairs(GridFrame.registeredFrames) do
					SetCornerSize(f)
				end
			end,
		},
		["offset1V"] = {
			type = "range",
			name = LGrid["Vertical"],
			width = "fill",
			order = 13,
			min = -20,
			max = 20,
			step = 1,
			get = function() return GridIndicatorExtraText.db.profile.offset1V end,
			set = function(_, v)
				GridIndicatorExtraText.db.profile.offset1V = v
				for _, f in pairs(GridFrame.registeredFrames) do
					SetCornerSize(f)
				end
			end,
		},
		["desc2"] = {
			type = "description",
			order = 21,
			width = "full",
			name = L["Offset"].." "..LGrid["Top Right"]..":",
		},
		["offset2H"] = {
			type = "range",
			name = LGrid["Horizontal"],
			order = 22,
			min = -20,
			max = 20,
			step = 1,
			get = function() return GridIndicatorExtraText.db.profile.offset2H end,
			set = function(_, v)
				GridIndicatorExtraText.db.profile.offset2H = v
				for _, f in pairs(GridFrame.registeredFrames) do
					SetCornerSize(f)
				end
			end,
		},
		["offset2V"] = {
			type = "range",
			name = LGrid["Vertical"],
			order = 23,
			min = -20,
			max = 20,
			step = 1,
			get = function() return GridIndicatorExtraText.db.profile.offset2V end,
			set = function(_, v)
				GridIndicatorExtraText.db.profile.offset2V = v
				for _, f in pairs(GridFrame.registeredFrames) do
					SetCornerSize(f)
				end
			end,
		},
		["desc3"] = {
			type = "description",
			order = 31,
			width = "full",
			name = L["Offset"].." "..LGrid["Bottom Left"]..":",
		},
		["offset3H"] = {
			type = "range",
			name = LGrid["Horizontal"],
			order = 32,
			min = -20,
			max = 20,
			step = 1,
			get = function() return GridIndicatorExtraText.db.profile.offset3H end,
			set = function(_, v)
				GridIndicatorExtraText.db.profile.offset3H = v
				for _, f in pairs(GridFrame.registeredFrames) do
					SetCornerSize(f)
				end
			end,
		},
		["offset3V"] = {
			type = "range",
			name = LGrid["Vertical"],
			order = 33,
			min = -20,
			max = 20,
			step = 1,
			get = function() return GridIndicatorExtraText.db.profile.offset3V end,
			set = function(_, v)
				GridIndicatorExtraText.db.profile.offset3V = v
				for _, f in pairs(GridFrame.registeredFrames) do
					SetCornerSize(f)
				end
			end,
		},
		["desc4"] = {
			type = "description",
			order = 41,
			width = "full",
			name = L["Offset"].." "..LGrid["Bottom Right"]..":",
		},
		["offset4H"] = {
			type = "range",
			name = LGrid["Horizontal"],
			order = 42,
			min = -20,
			max = 20,
			step = 1,
			get = function() return GridIndicatorExtraText.db.profile.offset4H end,
			set = function(_, v)
				GridIndicatorExtraText.db.profile.offset4H = v
				for _, f in pairs(GridFrame.registeredFrames) do
					SetCornerSize(f)
				end
			end,
		},
		["offset4V"] = {
			type = "range",
			name = LGrid["Vertical"],
			order = 43,
			min = -20,
			max = 20,
			step = 1,
			get = function() return GridIndicatorExtraText.db.profile.offset4V end,
			set = function(_, v)
				GridIndicatorExtraText.db.profile.offset4V = v
				for _, f in pairs(GridFrame.registeredFrames) do
					SetCornerSize(f)
				end
			end,
		},
		["header5"] = {
			type = "header",
			order = 50,
			width = "full",
			name = "",
		},
		["originalsize"] = {
			type = "toggle",
			name = L["Same settings as Grid"],
			order = 80,
			width = "double",
			get = function() return GridIndicatorExtraText.db.profile.OriginalSize end,
			set = function(_, v)
				GridIndicatorExtraText.db.profile.OriginalSize = v
				for _, f in pairs(GridFrame.registeredFrames) do
					SetCornerSize(f)
				end
				if configMode then
					GridIndicatorExtraText:ConfigMode()
				end
			end,
		},
		["originalopt"] = {
			name = " ",
			order = 85,
			type = "group",
			dialogInline = true,
			args = {
				["font"] = {
					type = "select",
					name = LGrid["Font"],
					desc = LGrid["Adjust the font settings"],
					order = 90,
					disabled = function() return GridIndicatorExtraText.db.profile.OriginalSize end,
					width = "double",
					values = media:HashTable("font"),
					dialogControl = "LSM30_Font",
					get = function() return GridIndicatorExtraText.db.profile.fontType end,
					set = function(_, v)
						GridIndicatorExtraText.db.profile.fontType = v
						for _, f in pairs(GridFrame.registeredFrames) do
							SetCornerSize(f)
						end
						if configMode then
							GridIndicatorExtraText:ConfigMode()
						end
					end,
				},
				["dummy1"] = {
					type = "description",
					order = 91,
					width = "full",
					name = "",
				},
				["fontSize"] = {
					type = "range",
					name = LGrid["Font Size"],
					desc = LGrid["Adjust the font size."],
					order = 100,
					disabled = function() return GridIndicatorExtraText.db.profile.OriginalSize end,
					width = "double",
					min = 5,
					max = 24,
					step = 1,
					get = function() return GridIndicatorExtraText.db.profile.fontSize end,
					set = function(_, v)
						GridIndicatorExtraText.db.profile.fontSize = v
						for _, f in pairs(GridFrame.registeredFrames) do
							SetCornerSize(f)
						end
						if configMode then
							GridIndicatorExtraText:ConfigMode()
						end
					end,
				},
				["dummy2"] = {
					type = "description",
					order = 101,
					width = "full",
					name = "",
				},
				["fontOutline"] = {
					type = "select",
					name = LGrid["Font Outline"],
					desc = LGrid["Adjust the font outline."],
					order = 110,
					disabled = function() return GridIndicatorExtraText.db.profile.OriginalSize end,
					width = "double",
					values = {
						NONE = LGrid["None"],
						OUTLINE = LGrid["Thin"],
						THICKOUTLINE = LGrid["Thick"] ,
					},
					get = function() return GridIndicatorExtraText.db.profile.fontOutline end,
					set = function(_, v)
						GridIndicatorExtraText.db.profile.fontOutline = v
						for _, f in pairs(GridFrame.registeredFrames) do
							SetCornerSize(f)
						end
						if configMode then
							GridIndicatorExtraText:ConfigMode()
						end
					end,
				},
				["dummy3"] = {
					type = "description",
					order = 111,
					width = "full",
					name = "",
				},
				["fontShadow"] = {
					type = "toggle",
					name = LGrid["Font Shadow"],
					desc = LGrid["Toggle the font drop shadow effect."],
					order = 120,
					disabled = function() return GridIndicatorExtraText.db.profile.OriginalSize end,
					width = "double",
					get = function() return GridIndicatorExtraText.db.profile.fontShadow end,
					set = function(_, v)
						GridIndicatorExtraText.db.profile.fontShadow = v
						for _, f in pairs(GridFrame.registeredFrames) do
							SetCornerSize(f)
						end
						if configMode then
							GridIndicatorExtraText:ConfigMode()
						end
					end,
				},
				["dummy4"] = {
					type = "description",
					order = 121,
					width = "full",
					name = "",
				},
				["textlength"] = {
					type = "range",
					name = L["Text Length"],
					order = 130,
					disabled = function() return GridIndicatorExtraText.db.profile.OriginalSize end,
					width = "double",
					min = 1,
					max = 12,
					step = 1,
					get = function() return GridIndicatorExtraText.db.profile.maxTextLength end,
					set = function(_, v)
						GridIndicatorExtraText.db.profile.maxTextLength = v
						for _, f in pairs(GridFrame.registeredFrames) do
							SetCornerSize(f)
						end
						if configMode then
							GridIndicatorExtraText:ConfigMode()
						end
					end,
				}
			}
		}
	}
}
Grid.options.args["GridIndicatorExtraText"] = options



local statusmap = GridFrame.db.profile.statusmap
if not statusmap["extraText1TL"] then
	statusmap["extraText1TL"] = {}
	statusmap["extraText2TR"] = {}
	statusmap["extraText3BL"] = {}
	statusmap["extraText4BR"] = {}
end



function GridIndicatorExtraText:OnInitialize()
	if not self.db then
		self.db = Grid.db:RegisterNamespace(self.moduleName, { profile = self.defaultDB or { } })
	end
	GridFrame:RegisterIndicator("extraText1TL", LGrid["Top Left"],     NewIndicator, ResetIndicator, SetIndicator, ClearIndicator)
	GridFrame:RegisterIndicator("extraText2TR", LGrid["Top Right"],    NewIndicator, ResetIndicator, SetIndicator, ClearIndicator)
	GridFrame:RegisterIndicator("extraText3BL", LGrid["Bottom Left"],  NewIndicator, ResetIndicator, SetIndicator, ClearIndicator)
	GridFrame:RegisterIndicator("extraText4BR", LGrid["Bottom Right"], NewIndicator, ResetIndicator, SetIndicator, ClearIndicator)

	hooksecurefunc(GridFrame, "UpdateOptionsMenu", self.ChangeOptionsMenu)
	GridIndicatorExtraText:ChangeOptionsMenu()
	GridIndicatorExtraText:CreateConfigFrame()
end



function GridIndicatorExtraText:OnEnable()
end



function GridIndicatorExtraText:OnDisable()
end



function GridIndicatorExtraText:Reset()
end



function GridIndicatorExtraText:ChangeOptionsMenu()
	if not GridIndicatorExtraText:IsEnabled() then return end
	if not Grid.options then return end
	if not Grid.options.args then return end
	local GridIndicator
	if     type(Grid.options.args.GridIndicator) == "table" then GridIndicator = "GridIndicator"
	elseif type(Grid.options.args.Indicators)    == "table" then GridIndicator = "Indicators" end
	if not Grid.options.args[GridIndicator] then return end
	if not Grid.options.args[GridIndicator].args then return end

	if not Grid.options.args[GridIndicator].args.extraText1TL then return end

	Grid.options.args[GridIndicator].args.GridIndicatorExtraText = {
		type = "group",
		icon = "Interface\\AddOns\\GridIndicatorExtraText\\GridIndicatorExtraText-icon-ALL",
		name = L["Extra Text"],
		order = 7.1,
		args = {
			["extraText1TL"] = Grid.options.args[GridIndicator].args.extraText1TL,
			["extraText2TR"] = Grid.options.args[GridIndicator].args.extraText2TR,
			["extraText3BL"] = Grid.options.args[GridIndicator].args.extraText3BL,
			["extraText4BR"] = Grid.options.args[GridIndicator].args.extraText4BR
		}
	}
	Grid.options.args[GridIndicator].args.GridIndicatorExtraText.args.extraText1TL.icon = "Interface\\AddOns\\GridIndicatorExtraText\\GridIndicatorExtraText-icon-TL"
	Grid.options.args[GridIndicator].args.GridIndicatorExtraText.args.extraText2TR.icon = "Interface\\AddOns\\GridIndicatorExtraText\\GridIndicatorExtraText-icon-TR"
	Grid.options.args[GridIndicator].args.GridIndicatorExtraText.args.extraText3BL.icon = "Interface\\AddOns\\GridIndicatorExtraText\\GridIndicatorExtraText-icon-BL"
	Grid.options.args[GridIndicator].args.GridIndicatorExtraText.args.extraText4BR.icon = "Interface\\AddOns\\GridIndicatorExtraText\\GridIndicatorExtraText-icon-BR"
	Grid.options.args[GridIndicator].args.GridIndicatorExtraText.args.extraText1TL.order = 1
	Grid.options.args[GridIndicator].args.GridIndicatorExtraText.args.extraText2TR.order = 2
	Grid.options.args[GridIndicator].args.GridIndicatorExtraText.args.extraText3BL.order = 3
	Grid.options.args[GridIndicator].args.GridIndicatorExtraText.args.extraText4BR.order = 4
	Grid.options.args[GridIndicator].args.extraText1TL = nil
	Grid.options.args[GridIndicator].args.extraText2TR = nil
	Grid.options.args[GridIndicator].args.extraText3BL = nil
	Grid.options.args[GridIndicator].args.extraText4BR = nil
end



function GridIndicatorExtraText:CreateConfigFrame()
	GridIndicatorExtraText.ConfigWarningFrame = CreateFrame("Frame", nil, UIParent)
	local frame = GridIndicatorExtraText.ConfigWarningFrame
	frame:EnableMouse(true)
	frame:SetMovable(true)
	frame:SetToplevel(true)
	frame:SetClampedToScreen(true)
	frame:SetHeight(100)
	frame:SetBackdrop({
		bgFile = "Interface\\DialogFrame\\UI-DialogBox-Background",
		edgeFile = "Interface\\DialogFrame\\UI-DialogBox-Border",
		tile = true, tileSize = 32, edgeSize = 32,
		insets = {left = 8, right = 8, top = 8, bottom = 8}
	})
	frame:SetPoint("LEFT", GridLayoutFrame, "RIGHT", 40, 0)
	frame:SetScript("OnMouseDown", function(self)
		self.isMoving = true
		self:StartMoving()
	end)
	frame:SetScript("OnMouseUp", function(self)
		if not self.isMoving then return end
		self.isMoving = nil
		self:StopMovingOrSizing()
	end)
	frame:Hide()

	local background = frame:CreateTexture(nil, "ARTWORK")
	background:SetPoint("TOP", 0, -15)
	background:SetTexture("Interface\\DialogFrame\\UI-Dialog-Icon-AlertNew")
	background:SetHeight(24)
	background:SetWidth(24)

	local txt = frame:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
	txt:SetPoint("TOP", background, "BOTTOM", 0, -5)
	txt:SetText("Grid - "..L["Extra Text"].." - "..L["Configuration Mode"])
	txt:SetTextColor(1, 1, 1, 1)

	local button = CreateFrame("Button", nil, frame, "UIPanelButtonTemplate")
	button:SetHeight(24)
	button:SetPoint("TOP", txt, "BOTTOM", 0, -5)
	button:SetText(CLOSE)
	button:SetScript("OnClick", function()
		configMode = false
		GridIndicatorExtraText:ConfigMode()
	end)

	local w = txt:GetStringWidth()
	button:SetWidth(w+50)
	frame:SetWidth(w+50+60)
end



function GridIndicatorExtraText:ConfigMode()
	if configMode then
		GridIndicatorExtraText.ConfigWarningFrame:Show()
		for _, frame in pairs(GridFrame.registeredFrames) do
			SetIndicator(frame.indicators.extraText1TL, {r=0.22, g=0.85, b=0.81, a=1}, "12345678901234567890")
			SetIndicator(frame.indicators.extraText2TR, {r=0.87, g=0.62, b=0.06, a=1}, "12345678901234567890")
			SetIndicator(frame.indicators.extraText3BL, {r=0.29, g=0.34, b=0.67, a=1}, "12345678901234567890")
			SetIndicator(frame.indicators.extraText4BR, {r=0.12, g=0.46, b=0.97, a=1}, "12345678901234567890")
		end
	else
		GridIndicatorExtraText.ConfigWarningFrame:Hide()
		for _, frame in pairs(GridFrame.registeredFrames) do
			ClearIndicator(frame.indicators.extraText1TL)
			ClearIndicator(frame.indicators.extraText2TR)
			ClearIndicator(frame.indicators.extraText3BL)
			ClearIndicator(frame.indicators.extraText4BR)
		end
	end
end